/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.framebuffer.FrameBufferPanel;

import java.awt.Color;
import javax.swing.JFrame;
import java.awt.BorderLayout;
import java.io.File;

/**

*/
public class InteractiveModels_R7 extends InteractiveAbstractClient_R7
{
   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
      Then this constructor instantiates the GUI.
   */
   public InteractiveModels_R7()
   {
      // Create the Scene object that we shall render.
      scene = new Scene();

      // Add four (empty) Positions to the Scene.
      scene.addPosition(new Position(), new Position(),
                        new Position(), new Position());

      // Push the Positions away from where the camera is.
      scene.getPosition(0).matrix = Matrix.translate(0, 0, pushback);
      scene.getPosition(1).matrix = Matrix.translate(0, 0, pushback);
      scene.getPosition(2).matrix = Matrix.translate(0, 0, pushback);
      scene.getPosition(3).matrix = Matrix.translate(0, 0, pushback);

      // Create several Model objects.
      modelArray.add( new ObjSimpleModel(new File("assets/apple.obj")) );
      modelArray.add( new ObjSimpleModel(new File("assets/cow.obj")) );
      modelArray.add( new ObjSimpleModel(new File("assets/galleon.obj")) );
      modelArray.add( new ObjSimpleModel(new File("assets/teapot.obj")) );
      modelArray.add( new ObjSimpleModel(new File("assets/cessna.obj")) );
      modelArray.add( new Sphere(1.0, 30, 30) );
      modelArray.add( new Cylinder(0.5, 1.0, 20, 20) );
      modelArray.add( new Torus(0.75, 0.25, 25, 25) );
      modelArray.add( new Cube2(15, 15, 15) );
      modelArray.add( new ObjSimpleModel(new File("assets/small_rhombicosidodecahedron.obj")) );
      final Model floor = new PanelXZ(-7, 7, -3, 1);  // floor
      final Model wall  = new PanelXY(-7, 7, -1, 3);  // wall
      final Model airplane = new ObjSimpleModel(new File("assets/cessna.obj"));

      // Give each model a random color.
      for (final Model m : modelArray)
      {
         ModelShading.setRandomColor(m);
      }
      ModelShading.setRandomColor(floor);
      ModelShading.setRandomColor(wall);
      ModelShading.setRandomColor(airplane);

      // Add four models to the Scene.
      scene.getPosition(0).setModel( modelArray.get(1) ); // cow
      scene.getPosition(1).setModel( wall );
      scene.getPosition(2).setModel( floor );
      scene.getPosition(3).setModel( airplane );
      currentModel = 1;

      // Position the wall, floor and plane.
      scene.getPosition(1).matrix.mult( Matrix.translate(0,  0, -3) );// wall
      scene.getPosition(2).matrix.mult( Matrix.translate(0, -1,  0) );// floor
      scene.getPosition(3).matrix.mult( Matrix.translate(3,  0,  0) );// plane

      xTranslation = new double[modelArray.size()];
      yTranslation = new double[modelArray.size()];
      zTranslation = new double[modelArray.size()];
      xRotation = new double[modelArray.size()];
      yRotation = new double[modelArray.size()];
      zRotation = new double[modelArray.size()];
      scale = new double[modelArray.size()];
      java.util.Arrays.fill(scale, 1.0);


      // Define initial dimensions for a FrameBuffer.
      final int width  = 1024;
      final int height = 1024;

      // Create a FrameBufferPanel that will hold a FrameBuffer.
      fbp = new FrameBufferPanel(width, height);
      fbp.getFrameBuffer().setBackgroundColorFB(Color.darkGray);
      fbp.getFrameBuffer().getViewport().setBackgroundColorVP(Color.black);

      // Create a JFrame that will hold the FrameBufferPanel.
      jf = new JFrame("Renderer 8");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    //jf.setLocationByPlatform(true);
      // Place the FrameBufferPanel in the JFrame.
      jf.getContentPane().add(fbp, BorderLayout.CENTER);
      jf.pack();
      jf.setVisible(true);

      // Create event handler objects for events from the JFrame.
      jf.addKeyListener(this);
      jf.addComponentListener(this);
   }


   @Override protected void setTransformations(final char c)
   {
      if ('=' == c)
      {
         scale[currentModel] = 1.0;
         xTranslation[currentModel] = 0.0;
         yTranslation[currentModel] = 0.0;
         zTranslation[currentModel] = 0.0;
         xRotation[currentModel] = 0.0;
         yRotation[currentModel] = 0.0;
         zRotation[currentModel] = 0.0;
      }
      else if ('s' == c) // Scale the model 10% smaller.
      {
         scale[currentModel] /= 1.1;
      }
      else if ('S' == c) // Scale the model 10% larger.
      {
         scale[currentModel] *= 1.1;
      }
      else if ('x' == c)
      {
         xTranslation[currentModel] -= 0.1;
      }
      else if ('X' == c)
      {
         xTranslation[currentModel] += 0.1;
      }
      else if ('y' == c)
      {
         yTranslation[currentModel] -= 0.1;
      }
      else if ('Y' == c)
      {
         yTranslation[currentModel] += 0.1;
      }
      else if ('z' == c)
      {
         zTranslation[currentModel] -= 0.1;
      }
      else if ('Z' == c)
      {
         zTranslation[currentModel] += 0.1;
      }
      else if ('u' == c)
      {
         xRotation[currentModel] -= 2.0;
      }
      else if ('U' == c)
      {
         xRotation[currentModel] += 2.0;
      }
      else if ('v' == c)
      {
         yRotation[currentModel] -= 2.0;
      }
      else if ('V' == c)
      {
         yRotation[currentModel] += 2.0;
      }
      else if ('w' == c)
      {
         zRotation[currentModel] -= 2.0;
      }
      else if ('W' == c)
      {
         zRotation[currentModel] += 2.0;
      }

      // Set the model-to-view transformation matrix.
      // The order of the transformations is very important!
      final Position model_p = scene.getPosition(0);
      // Push the model away from where the camera is.
      model_p.matrix = Matrix.translate(0, 0, pushback);
      // Move the model relative to its new position.
      model_p.matrix.mult( Matrix.translate(xTranslation[currentModel],
                                            yTranslation[currentModel],
                                            zTranslation[currentModel]) );
      model_p.matrix.mult( Matrix.rotateX(xRotation[currentModel]) );
      model_p.matrix.mult( Matrix.rotateY(yRotation[currentModel]) );
      model_p.matrix.mult( Matrix.rotateZ(zRotation[currentModel]) );
      model_p.matrix.mult( Matrix.scale(scale[currentModel]) );
   }


   @Override protected void displayMatrix(final char c)
   {
      if ('m'==c||'='==c||'/'==c||'?'==c
        ||'s'==c||'x'==c||'y'==c||'z'==c||'u'==c||'v'==c||'w'==c
        ||'S'==c||'X'==c||'Y'==c||'Z'==c||'U'==c||'V'==c||'W'==c)
      {
         System.out.println("xRot = " + xRotation[0]
                        + ", yRot = " + yRotation[0]
                        + ", zRot = " + zRotation[0]);
         System.out.print( scene.getPosition(0).matrix );
      }
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new InteractiveModels_R7()
      );
   }
}
