/*

*/

import renderer.scene.*;
import renderer.framebuffer.FrameBufferPanel;

import java.awt.Color;
import javax.swing.JFrame;
import java.awt.BorderLayout;

/**
<pre>{@code
                  y       |
                  |       |
                  |       |
                  | v[1]  |
                1 +       |   /
                  |       |  /
                  |       | /
                  |       |/
                  |    -1 +--------------- image plane
                  |      /
                  |     /
                  |    /
                  |   /
                  |  /
                  | /
                  |/                v[0]
             v[2] +-----------------+------> x
                 /0                 1
                /
               /
              /
           1 +
            /
           z
}</pre>
   Render a wireframe triangle. This is just about the
   simplest possible model. It is useful for debugging.
*/
public class InteractiveTriangle_R7 extends InteractiveAbstractClient_R7
{
   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
      Then this constructor instantiates the GUI.
   */
   public InteractiveTriangle_R7()
   {
      // Create the Scene object that we shall render.
      scene = new Scene();

      // Create a Model object to hold the geometry.
      final Model model = new Model("triangle");
      modelArray.add( model );

      // Create a Position for this Model.
      final Position position = new Position(modelArray.get(0));

      // Add the Position (and its Model) to the Scene.
      scene.addPosition(position);

      // Push the Position away from where the camera is.
      pushback = -1.0;
      position.matrix = Matrix.translate(0, 0, pushback);

      // Create the geometry for the Model.
      model.addVertex(new Vertex(1.0, 0.0, 0.0),
                      new Vertex(0.0, 1.0, 0.0));
      Vertex v2 =     new Vertex(0.0, 0.0, 0.0);
    //Vertex v2 =     new Vertex(0.0, 0.0, 1.0);  // lands right on the camera
      model.addVertex(v2);

      // Create a different color for each vertex.
      model.addColor(new Color(255,  0,   0 ),  // red
                     new Color( 0,  255,  0 ),  // green
                     new Color( 0,   0,  255)); // blue

      // Add the geometry with colors to the Model.
      model.addLineSegment(new LineSegment(0, 1, 0, 1),
                           new LineSegment(1, 2, 1, 2),
                           new LineSegment(2, 0, 2, 0));

      System.out.println(model);


      // Define initial dimensions for a FrameBuffer.
      final int width  = 512;
      final int height = 512;

      // Create a FrameBufferPanel that will hold a FrameBuffer.
      fbp = new FrameBufferPanel(width, height);
      fbp.getFrameBuffer().setBackgroundColorFB(Color.darkGray);
      fbp.getFrameBuffer().getViewport().setBackgroundColorVP(Color.black);

      // Create a JFrame that will hold the FrameBufferPanel.
      jf = new JFrame("Renderer 8");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    //jf.setLocationByPlatform(true);
      // Place the FrameBufferPanel in the JFrame.
      jf.getContentPane().add(fbp, BorderLayout.CENTER);
      jf.pack();
      jf.setVisible(true);

      // Create event handler objects for events from the JFrame.
      jf.addKeyListener(this);
      jf.addComponentListener(this);
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new InteractiveTriangle_R7()
      );
   }
}
